import numpy as np
from scipy import sparse
from numpy.testing import assert_array_almost_equal
import nose
from nose.tools import assert_almost_equal

from iced.normalization import ICE_normalization
from iced.normalization import SCN_normalization


def test_ICE_normalization():
    n = 100
    random_state = np.random.RandomState(seed=42)
    X = random_state.randint(0, 100, size=(n, n))
    X = X + X.T
    normed_X = ICE_normalization(X, eps=1e-10, max_iter=1000000)
    normed = normed_X.sum(axis=1)
    assert_array_almost_equal(normed / normed.mean(), np.ones((len(X), )),
                              decimal=0)

    normed_X, bias = ICE_normalization(X, eps=1e-10, max_iter=100000,
                                       output_bias=True)
    assert_array_almost_equal(normed_X, X / (bias.T * bias), 6)


def test_sparse_ICE_normalization():
    n = 100
    random_state = np.random.RandomState(seed=42)
    X = random_state.randint(0, 100, size=(n, n))

    thres = (random_state.rand(n, n) > 0.5).astype(bool)

    X[thres] = 0
    X = X + X.T
    sparse_X = sparse.csr_matrix(X)
    true_normed_X = ICE_normalization(X, eps=1e-10, max_iter=10)
    normed_X = ICE_normalization(sparse_X, eps=1e-10, max_iter=10)
    assert_array_almost_equal(X, sparse_X.todense())
    assert_array_almost_equal(true_normed_X, np.array(normed_X.todense()))


def test_sparse_ICE_normalization_triu():
    n = 100
    random_state = np.random.RandomState(seed=42)
    X = random_state.randint(0, 100, size=(n, n))

    thres = (random_state.rand(n, n) > 0.5).astype(bool)
    X[thres] = 0
    X = X + X.T
    sparse_X = sparse.triu(X)
    true_normed_X, true_biases = ICE_normalization(
        X, eps=1e-10, max_iter=10, output_bias=True)
    true_normed_X = np.triu(true_normed_X)

    normed_X_sparse, biases_sparse = ICE_normalization(
        sparse_X, eps=1e-10, max_iter=100,
        output_bias=True)
    normed_X_dense, biases_dense = ICE_normalization(
        np.triu(X), eps=1e-10, max_iter=100,
        output_bias=True)

    # The sparse and dense version are going to be equal up to a constant
    # factor
    assert_array_almost_equal(normed_X_dense,
                              np.array(normed_X_sparse.toarray()))

    normed_X_sparse *= true_normed_X.mean() / normed_X_sparse.mean()
    normed_X_dense *= true_normed_X.mean() / normed_X_dense.mean()

    assert_array_almost_equal(true_normed_X,
                              np.array(normed_X_sparse.todense()))
    assert_array_almost_equal(true_normed_X, normed_X_dense)

    total_counts = 5000
    normed_X = ICE_normalization(sparse_X, eps=1e-10,
                                 total_counts=total_counts)
    assert_almost_equal(normed_X.sum(), total_counts)


def test_SCN_normalization():
    n = 100
    random_state = np.random.RandomState(seed=42)
    X = random_state.randint(0, 100, size=(n, n))

    normed_X = SCN_normalization(X)
    assert_array_almost_equal(np.sqrt((normed_X ** 2).sum(axis=1)),
                              np.ones((len(X), )))


if __name__ == "__main__":
    nose.runmodule(argv=['-s', '--with-doctest'], exit=False)





def _check_squared_array(X):
    """
    Check whether arrays are squared

    Parameters
    ----------
    X : ndarray

    Returns
    -------
    X
    """
    if len(X.shape) != 2:
        raise ValueError(
            "The ndarray has %d dimension. 2D array is expected." %
            len(X.shape))

    if X.shape[0] != X.shape[1]:
        raise ValueError(
            "The ndarray is of shape (%d, %d). Squared array is expected." %
            (X.shape[0], X.shape[1]))

    return X



import numpy as np
from scipy import sparse
from scipy import optimize
from sklearn.metrics import euclidean_distances
from sklearn.utils import check_random_state

"""Utility method for the pyipopt solver, for the Poisson Model
"""

VERBOSE = False
niter = 0


def poisson_exp(X, counts, alpha, bias=None,
                beta=None, use_empty_entries=True):
    """
    Computes the log likelihood of the poisson exponential model.

    Parameters
    ----------
    X : ndarray
        3D positions

    counts : n * n ndarray
        of interaction frequencies

    alpha : float
        parameter of the expential law

    beta : float, optional, default None
        onstant. If is set to None, it will be computed using the maximum log
        likelihood knowing alpha.

    use_empty_entries : boolean, optional, default False
        whether to use zeroes entries as information or not

    Returns
    -------
    ll : float
        log likelihood
    """
    if VERBOSE:
        print "Poisson power law model : computation of the log likelihood"
    if bias is None:
        bias = np.ones((counts.shape[0], 1))
    if sparse.issparse(counts):
        ll = _poisson_exp_sparse(X, counts, alpha, bias=bias, beta=beta,
                                 use_empty_entries=use_empty_entries)
    else:
        ll = _poisson_exp_dense(X, counts, alpha, bias=bias, beta=beta,
                                use_empty_entries=use_empty_entries)
    return ll


def _poisson_exp_dense(X, counts, alpha, bias,
                       beta=None, use_empty_entries=False):
    m, n = X.shape
    d = euclidean_distances(X)
    if use_empty_entries:
        mask = (np.invert(np.tri(m, dtype=np.bool)))
    else:
        mask = np.invert(np.tri(m, dtype=np.bool)) & (counts != 0) & (d != 0)

    bias = bias.reshape(-1, 1)
    if beta is None:
        beta = counts[mask].sum() / (
            (d[mask] ** alpha) * (bias * bias.T)[mask]).sum()

    g = beta * d ** alpha
    g *= bias * bias.T
    g = g[mask]

    ll = counts[mask] * np.log(beta) + \
        alpha * counts[mask] * np.log(d[mask]) + \
        counts[mask] * np.log(bias * bias.T)[mask]
    ll -= g
    # We are trying to maximise, so we need the opposite of the log likelihood
    if np.isnan(ll.sum()):
        raise ValueError("Objective function is Not a Number")
    return - ll.sum()


def _poisson_exp_sparse(X, counts, alpha, bias,
                        beta=None, use_empty_entries=False):
    m, n = X.shape
    # if X is big this is going to take too much ram.
    # XXX We should create a sparse matrix containing the same entries as the
    # contact count matrix.

    d = np.sqrt(((X[counts.row] - X[counts.col])**2).sum(axis=1))
    if use_empty_entries:
        raise NotImplementedError

    bias = bias.flatten()
    if beta is None:
        beta = counts.sum() / (
            (d ** alpha) * bias[counts.row] * bias[counts.col]).sum()

    g = beta * d ** alpha * \
        bias[counts.row] * bias[counts.col]
    ll = (counts.data * np.log(beta) + alpha * counts.data * np.log(d) +
          counts.data * np.log(bias[counts.row] * bias[counts.col]))
    ll -= g
    if np.isnan(ll.sum()):
        raise ValueError("Objective function is Not a Number")
    return -ll.sum()


def gradient_poisson_exp(X, counts, alpha, bias=None,
                         beta=None, use_empty_entries=True):
    """
    Computes the gradient of the log likelihood of the gradient in alpha and
    beta

    Parameters
    ----------
    X: ndarray
        3D positions

    counts: n * n ndarray
        of interaction frequencies

    alpha: float
        parameter of the expential law

    beta: float
        constant

    use_empty_entries: boolean, optional, default: False
        whether to use the zeros entries as information or not.

    Returns
    -------
    grad_alpha, grad_beta: float, float
        The value of the gradient in alpha and the value of the gradient in
        beta
    """

    if VERBOSE:
        print "Poisson exponential model : computation of the gradient"
    if bias is None:
        bias = np.ones((counts.shape[0], 1))
    if sparse.issparse(counts):
        return _gradient_poisson_exp_sparse(
            X, counts, alpha, bias, beta,
            use_empty_entries=use_empty_entries)
    else:
        return _gradient_poisson_exp_dense(
            X, counts, alpha, bias, beta,
            use_empty_entries=use_empty_entries)


def _gradient_poisson_exp_dense(X, counts, alpha, bias, beta,
                                use_empty_entries=True):
    m, n = X.shape

    d = euclidean_distances(X)

    bias = bias.reshape(-1, 1)

    if use_empty_entries:
        mask = np.invert(np.tri(m, dtype=np.bool))
    else:
        mask = np.invert(np.tri(m, dtype=np.bool)) & (counts != 0)

    beta = counts[mask].sum() / (
        (d[mask] ** alpha) * (bias * bias.T)[mask]).sum()

    grad_alpha = - beta * \
        ((bias * bias.T)[mask] * d[mask] ** alpha * np.log(d[mask])).sum() \
        + (counts[mask] * np.log(d[mask])).sum()
    return - np.array([grad_alpha])


def _gradient_poisson_exp_sparse(X, counts, alpha, bias, beta,
                                 use_empty_entries=True):
    m, n = X.shape
    bias = bias.flatten()

    if use_empty_entries:
        raise NotImplementedError
    d = np.sqrt(((X[counts.row] - X[counts.col])**2).sum(axis=1))

    beta = counts.sum() / (
        (d ** alpha) * bias[counts.row] * bias[counts.col]).sum()

    grad_alpha = - beta * (bias[counts.row] * bias[counts.col] * d ** alpha *
                           np.log(d)).sum() \
        + (counts.data *
           np.log(d)).sum()
    return - np.array([grad_alpha])


def eval_f(x, user_data=None):
    """
    Evaluate the objective function.

    This computes the stress
    """
    if VERBOSE:
        print "Poisson exponential model : eval_f"

    m, n, counts, X, bias, use_empty_entries = user_data
    X = X.reshape((m, n))
    tmp = poisson_exp(X, counts, x[0], bias=bias,
                      use_empty_entries=use_empty_entries)
    return tmp


def eval_f_X(X, user_data=None):
    """
    Evaluate the objective function.

    This computes the stress

    Parameters
    ----------
    X: ndarray, shape m * n
        3D configuration

    user_data: optional, default=None
        m, n, counts, alpha, beta

    Returns
    -------
    loglikelihood of the model.
    """
    if VERBOSE:
        print "Poisson exponential model : computation of the eval_f"

    m, n, counts, alpha, beta, d = user_data
    X = X.reshape((m, n))
    tmp = poisson_exp(X, counts, alpha, beta)
    return tmp


def eval_grad_f(x, user_data=None):
    """
    Evaluate the gradient of the function in alpha
    """
    if VERBOSE:
        print "Poisson exponential model : eval_grad_f (evaluation in alpha)"

    m, n, counts, X, bias, use_empty_entries = user_data
    X = X.reshape((m, n))
    tmp = gradient_poisson_exp(X, counts, x[0], bias=bias, beta=None,
                               use_empty_entries=use_empty_entries)

    return tmp


def eval_grad_f_X(X, user_data=None):
    """
    Evaluate the gradient of the function in X
    """
    global niter
    niter += 1
    if not niter % 10:
        X.dump('%d.sol.npy' % niter)

    if VERBOSE:
        print "Poisson exponential model : eval_grad_f_X (evaluation in f X)"

    m, n, counts, alpha, beta, d = user_data
    X = X.reshape((m, n))
    dis = euclidean_distances(X)

    tmp = X.repeat(m, axis=0).reshape((m, m, n))
    dif = tmp - tmp.transpose(1, 0, 2)
    dis = dis.repeat(n).reshape((m, m, n))
    counts = counts.repeat(n).reshape((m, m, n))

    grad = - alpha * beta * dif / dis * (dis ** (alpha - 1)) + \
        counts * alpha * dif / (dis ** 2)
    grad[np.isnan(grad)] = 0
    return - grad.sum(1)


def eval_stress(X, user_data=None):
    """
    """
    if VERBOSE:
        print "Computing stress: eval_stress"
    m, n, distances, alpha, beta, d = user_data
    X = X.reshape((m, n))
    dis = euclidean_distances(X)
    stress = ((dis - distances) ** 2)[distances != 0].sum()
    return stress


def eval_grad_stress(X, user_data=None):
    """
    Compute the gradient of the stress
    """
    if VERBOSE:
        print 'Compute the gradient of the stress: eval_grad_stress'
    m, n, distances, alpha, beta, d = user_data
    X = X.reshape((m, n))
    tmp = X.repeat(m, axis=0).reshape((m, m, n))
    dif = tmp - tmp.transpose(1, 0, 2)
    dis = euclidean_distances(X).repeat(3, axis=1).flatten()
    distances = distances + distances.T
    distances = distances.repeat(3, axis=1).flatten()
    grad = 2 * dif.flatten() * (dis - distances) / dis
    grad[(distances == 0) | np.isnan(grad)] = 0
    return grad.reshape((m, m, n)).sum(axis=1)


def eval_h(x, lagrange, obj_factor, flag, user_data=None):
    """
    """
    return False


def _estimate_beta(counts, X, alpha=-3, bias=None):
    m, n = X.shape
    if bias is None:
        bias = np.ones((counts.shape[0], 1))
    if sparse.issparse(counts):
        counts = counts.tocoo()
        dis = np.sqrt(((X[counts.row] - X[counts.col])**2).sum(axis=1))
        bias = bias.flatten()
        beta = counts.sum() / (
            (dis ** alpha) * bias[counts.row] * bias[counts.col]).sum()

    else:
        dis = euclidean_distances(X)
        mask = np.invert(np.tri(m, dtype=np.bool)) & (counts != 0) & (dis != 0)
        beta = counts[mask].sum() / (
            (dis[mask] ** alpha) * (bias * bias.T)[mask]).sum()

    return beta


def estimate_alpha_beta(counts, X, bias=None, ini=None, verbose=0,
                        use_empty_entries=False, random_state=None):
    """
    Estimate the parameters of g

    Parameters
    ----------
    counts: ndarray

    use_empty_entries: boolean, optional, default: True
        whether to use zeroes entries as information or not

    """

    m, n = X.shape
    bounds = np.array(
        [[-100, 1e-2]])

    random_state = check_random_state(random_state)
    if ini is None:
        ini = - random_state.randint(1, 100, size=(2, )) + \
            random_state.rand(1)
    data = (m, n, counts, X, bias,
            use_empty_entries)

    results = optimize.fmin_l_bfgs_b(
        eval_f,
        ini[0],
        eval_grad_f,
        (data, ),
        bounds=bounds,
        iprint=1,
        maxiter=1000,
        )

    beta = _estimate_beta(counts, X, alpha=results[0], bias=bias)
    return results[0], beta



import numpy as np
from scipy import special
from scipy import sparse
from scipy import optimize
from sklearn.utils import check_random_state
from .utils import ConstantDispersion


def negative_binomial_obj(X, counts, alpha=-3., beta=1., bias=None,
                          dispersion=None,
                          use_zero_counts=False):
    if bias is None:
        bias = np.ones((counts.shape[0], 1))

    if dispersion is None:
        dispersion = ConstantDispersion()

    if sparse.issparse(counts):
        return _negative_binomial_obj_sparse(
            X, counts, beta=beta, bias=bias,
            alpha=alpha, dispersion=dispersion,
            use_zero_counts=use_zero_counts)
    else:
        raise NotImplementedError
        return _negative_binomial_obj_dense(
            X, counts, beta=beta, bias=bias,
            alpha=alpha, dispersion=dispersion,
            use_zero_counts=use_zero_counts)


def _negative_binomial_obj_dense(X, counts, alpha=-3, beta=1, dispersion=None,
                                 bias=None,
                                 use_zero_counts=False):
    if use_zero_counts:
        raise NotImplementedError


def _negative_binomial_obj_sparse(X, counts, alpha=-3, beta=1., bias=None,
                                  dispersion=None,
                                  use_zero_counts=False):
    if use_zero_counts:
        raise NotImplementedError

    bias = bias.flatten()

    dis = np.sqrt(((X[counts.row] - X[counts.col])**2).sum(axis=1))
    fdis = bias[counts.row] * bias[counts.col] * beta * dis ** alpha

    d = dispersion.predict(fdis)
    if np.any(d < 0):
        print("Negative dispersion")

    obj = - special.gammaln(counts.data + d).sum()
    obj += special.gammaln(d).sum()
    obj -= (counts.data * np.log(fdis)).sum()
    obj -= (d * np.log(d)).sum()
    obj += ((counts.data + d) * np.log(d + fdis)).sum()
    return obj.sum()


def eval_f(x, user_data=None):
    """
    Evaluate the objective function.
    """
    m, n, counts, X, dispersion, bias, use_zero_counts = user_data
    X = X.reshape((m, n))
    return negative_binomial_obj(X, counts, alpha=x[0], beta=x[1],
                                 dispersion=dispersion,
                                 bias=bias,
                                 use_zero_counts=use_zero_counts)


def negative_binomial_gradient(X, counts, alpha=-3, beta=1, bias=None,
                               dispersion=None,
                               use_zero_counts=False):
    if bias is None:
        bias = np.ones((counts.shape[0], 1))

    if dispersion is None:
        dispersion = ConstantDispersion()

    if sparse.issparse(counts):
        return _negative_binomial_gradient_sparse(
            X, counts, beta=beta, bias=bias,
            alpha=alpha, dispersion=dispersion,
            use_zero_counts=use_zero_counts)
    else:
        raise NotImplementedError
        return _negative_binomial_gradient_dense(
            X, counts, beta=beta, bias=bias,
            alpha=alpha, dispersion=dispersion,
            use_zero_counts=use_zero_counts)


def _negative_binomial_gradient_dense(X, counts, alpha=-3, beta=1,
                                      dispersion=None,
                                      bias=None,
                                      use_zero_counts=False):
    if use_zero_counts:
        raise NotImplementedError


def _negative_binomial_gradient_sparse(X, counts, alpha=-3, beta=1.,
                                       dispersion=None,
                                       bias=None,
                                       use_zero_counts=False):
    if use_zero_counts:
        raise NotImplementedError
    bias = bias.flatten()

    dis = np.sqrt(((X[counts.row] - X[counts.col])**2).sum(axis=1))
    fdis = bias[counts.row] * bias[counts.col] * beta * dis ** alpha

    d = dispersion.predict(fdis)

    if np.any(d < 0):
        print("Negative dispersion")

    d_prime = dispersion.derivate(fdis) * fdis * np.log(dis)

    grad = - (special.digamma(counts.data + d) * d_prime).sum()
    grad += (special.digamma(d) * d_prime).sum()
    grad -= (counts.data * np.log(dis)).sum()
    grad -= (d_prime * (np.log(d) + 1)).sum()
    grad += (d_prime * np.log(d + fdis)).sum()
    grad += ((counts.data + d) / (d + fdis) *
             (d_prime + fdis * np.log(dis))).sum()

    d_prime = dispersion.derivate(fdis) * fdis / beta

    grad_beta = - (special.digamma(counts.data + d) * d_prime).sum()
    grad_beta += (special.digamma(d) * d_prime).sum()
    grad_beta -= (d_prime * (np.log(d) + 1)).sum()
    grad_beta -= (counts.data / beta).sum()
    grad_beta += (d_prime * np.log(d + fdis)).sum()
    grad_beta += (
        (counts.data + d) / (d + fdis) * (d_prime + fdis / beta)).sum()

    return np.array([grad, grad_beta])


def eval_grad_f(x, user_data=None):
    """
    Evaluate the gradient of the function in alpha
    """
    m, n, counts, X, dispersion, bias, use_zero_counts = user_data
    X = X.reshape((m, n))
    return negative_binomial_gradient(X, counts, alpha=x[0], beta=x[1],
                                      dispersion=dispersion,
                                      bias=bias,
                                      use_zero_counts=use_zero_counts)


def estimate_alpha_beta(counts, X, ini=None, dispersion=None, bias=None,
                        verbose=0,
                        random_state=None, maxiter=10000):
    """
    Estimate the parameters of g

    Parameters
    ----------
    counts: ndarray

    """
    m, n = X.shape
    bounds = np.array(
        [[-100, 1e-2],
         [1e-2, 10000]])

    random_state = check_random_state(random_state)
    if ini is None:
        ini = 5 * random_state.rand(2)
        ini[0] *= -1.
        ini = ini.astype(float)

    data = (m, n, counts, X, dispersion, bias,
            False)

    results = optimize.fmin_l_bfgs_b(
        eval_f,
        ini.flatten(),
        eval_grad_f,
        (data, ),
        bounds=bounds,
        iprint=1,
        maxiter=maxiter
        )
    return results[0]



import numpy as np


def write(X, filename, lengths=None,
          resolution=50000,
          copy=True):
    """
    Write the similarity matrix to a file

    Parameters
    ----------
    X: ndarray,
    """
    if copy:
        X = X.copy()
    X = X.astype(float)

    output_file = open(filename, 'w')

    tr = X.sum(axis=0) == 0
    if tr.sum() != 0:
        X[tr, :] = np.nan
        X[:, tr] = np.nan

    for i in range(len(X)):
        chr1, loc1 = _get_index(i, lengths)
        if loc1 > lengths[chr1 - 1]:
            raise ValueError("Problem !")
        loc1 *= resolution
        loc1 += 1
        for j in range(len(X)):
            if i > j:
                continue
            chr2, loc2 = _get_index(j, lengths)
            if loc2 > lengths[chr2 - 1]:
                raise ValueError("Problem !")
            loc2 *= resolution
            loc2 += 1

            if chr2 == chr1:
                dis = loc2 - loc1
            else:
                dis = 0

            if not X[i, j] or np.isnan(X[i, j]):
                continue

            output_file.write('%d\t%d\t%d\t%d\t%d\t%f\t0\t0\n' %
                              (chr1, loc1, chr2, loc2, dis, X[i, j]))

    output_file.close()


def _get_index(i, lengths):
    """
    From an index, return the chromosome number and the loci number

    Parameters
    ----------
    i : integer
        index in the full matrix

    lengths : ndarray (n, )

    Returns
    -------
    tuple : (c, l)
        chromosome number, locus
    """
    length_cum = lengths.cumsum()
    c = (length_cum > i).argmax() + 1

    if c > 1:
        length = i - length_cum[c - 2]
    else:
        length = i
    if len(lengths) > 1 and length > lengths[c - 1]:
        raise ValueError("there's a problem")

    if c > len(lengths):
        raise ValueError("There's a problem")
    return c, length



import numpy as np
from scipy import sparse
from nose.tools import assert_almost_equal
from numpy.testing import assert_array_almost_equal
from sklearn.metrics import euclidean_distances
from pastis.optimization import negative_binomial


def test_negative_binomial_obj_sparse():
    n = 10
    random_state = np.random.RandomState(42)
    X = random_state.rand(n, 3)
    counts = random_state.randn(n, n)
    counts = np.triu(counts)
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    # obj_dense = negative_binomial.negative_binomial_obj(X, counts)
    obj_sparse = negative_binomial.negative_binomial_obj(
        X, sparse.coo_matrix(counts))

    # assert_almost_equal(obj_dense, obj_sparse)


def test_negative_binomial_obj_sparse_biased():
    random_state = np.random.RandomState(seed=42)
    n = 50
    X = random_state.rand(n, 3)
    bias = 0.1 + random_state.rand(n)
    bias = bias.reshape(n, 1)

    counts = euclidean_distances(X)**(-3)
    counts[np.isinf(counts) | np.isnan(counts)] = 0
    counts *= bias * bias.T

    counts_dense = np.triu(counts)
    counts_sparse = sparse.coo_matrix(np.triu(counts))
    #exp_dense = negative_binomial.negative_binomial_obj(
    #    X, counts_dense, -3, bias=bias)
    exp_sparse = negative_binomial.negative_binomial_obj(
        X, counts_sparse, -3, bias=bias)
    #assert_almost_equal(exp_dense, exp_sparse)


def test_negative_binomial_grad_sparse():
    n = 10
    random_state = np.random.RandomState(seed=42)
    X = random_state.rand(n, 3)

    dis = euclidean_distances(X)
    alpha, beta = -3, 1

    counts = beta * dis ** alpha
    counts = np.triu(counts)
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    counts = sparse.coo_matrix(counts)

    # grad_dense = negative_binomial.negative_binomial_gradient(X, counts)
    grad_sparse = negative_binomial.negative_binomial_gradient(
        X, sparse.coo_matrix(counts))
    # assert_array_almost_equal(grad_dense, grad_sparse)
    assert_array_almost_equal(np.zeros(grad_sparse.shape),
                              grad_sparse)


def test_negative_binomial_grad_biased():
    random_state = np.random.RandomState(seed=42)
    n = 50
    X = random_state.rand(n, 3)
    bias = 0.1 + random_state.rand(n)
    bias = bias.reshape(n, 1)

    counts = euclidean_distances(X)**(-3)
    counts[np.isinf(counts) | np.isnan(counts)] = 0
    counts *= bias * bias.T

    counts_dense = counts
    counts_sparse = sparse.coo_matrix(np.triu(counts))
    # grad_dense = negative_binomial.negative_binomial_gradient(
    #    X, counts_dense, -3,
    #    bias=bias,
    #    beta=1)
    grad_sparse = negative_binomial.negative_binomial_gradient(
        X, counts_sparse, -3,
        bias=bias,
        beta=1)
    assert_array_almost_equal(np.array([0, 0.]), grad_sparse)
    # assert_array_almost_equal(grad_dense, grad_sparse)


def test_estimate_alpha_beta_biased():
    n = 100
    random_state = np.random.RandomState(seed=42)
    X = random_state.rand(n, 3)
    dis = euclidean_distances(X)
    bias = 0.1 + random_state.rand(n)
    bias = bias.reshape(n, 1)
    beta_true, alpha_true = 2., -3.5
    counts = beta_true * dis ** alpha_true
    counts *= bias * bias.T
    counts[np.isnan(counts) | np.isinf(counts)] = 0
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    counts = sparse.coo_matrix(np.triu(counts))

    alpha, beta = negative_binomial.estimate_alpha_beta(
        counts, X, bias=bias, random_state=random_state)
    assert_almost_equal(alpha_true, alpha, 3)
    assert_almost_equal(beta_true, beta, 3)

    alpha, beta = negative_binomial.estimate_alpha_beta(
        sparse.coo_matrix(counts), X, bias=bias,
        random_state=random_state)
    assert_almost_equal(alpha_true, alpha, 5)
    assert_almost_equal(beta_true, beta, 5)


def test_estimate_alpha_beta():
    n = 100
    random_state = np.random.RandomState(seed=42)
    X = 5 * random_state.rand(n, 3)
    dis = euclidean_distances(X)
    beta_true, alpha_true = 2., -3.5
    counts = beta_true * dis ** alpha_true
    counts[np.isnan(counts) | np.isinf(counts)] = 0
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    counts = sparse.coo_matrix(np.triu(counts))

    alpha, beta = negative_binomial.estimate_alpha_beta(
        sparse.coo_matrix(counts), X, random_state=random_state)
    assert_almost_equal(alpha_true, alpha, 3)
    assert_almost_equal(beta_true, beta, 4)



import numpy as np
from scipy import sparse
from ._normalization_ import _update_normalization_csr
from .utils import is_symetric_or_tri, is_tri


def ICE_normalization(X, SS=None, max_iter=3000, eps=1e-4, copy=True,
                      norm='l1', verbose=0, output_bias=False,
                      total_counts=None, counts_profile=None):
    """
    ICE normalization

    The imakaev normalization of Hi-C data consists of iteratively estimating
    the bias such that all the rows and columns (ie loci) have equal
    visibility.

    Parameters
    ----------
    X : ndarray or sparse array (n, n)
        raw interaction frequency matrix

    max_iter : integer, optional, default: 3000
        Maximum number of iteration

    eps : float, optional, default: 1e-4
        the relative increment in the results before declaring convergence.

    copy : boolean, optional, default: True
        If copy is True, the original data is not modified.

    norm : string, optional, default: l1
        If set to "l1", will compute the ICE algorithm of the paper. Else, the
        algorithm is adapted to use the l2 norm, as suggested in the SCN
        paper.

    output_bias : boolean, optional, default: False
        whether to output the bias vector.

    total_counts : float, optional, default: None
        the total number of contact counts that the normalized matrix should
        contain. If set to None, the normalized contact count matrix will be
        such that the total number of contact counts equals the initial number
        of interactions.

    Returns
    -------
    X, (bias) : ndarray (n, n)
        Normalized IF matrix and bias of output_bias is True

    Example
    -------
    .. plot:: examples/normalization/plot_ICE_normalization.py
    """
    if copy:
        X = X.copy()

    if sparse.issparse(X):
        if not sparse.isspmatrix_csr(X):
            X = sparse.csr_matrix(X, dtype="float")
        X.sort_indices()
    else:
        X[np.isnan(X)] = 0
    X = X.astype('float')

    m = X.shape[0]
    is_symetric_or_tri(X)
    old_dbias = None
    bias = np.ones((m, 1))
    _is_tri = is_tri(X)

    if total_counts is None:
        total_counts = X.sum()
    for it in np.arange(max_iter):
        if norm == 'l1':
            # Actually, this should be done if the matrix is diag sup or diag
            # inf
            if _is_tri:
                sum_ds = X.sum(axis=0) + X.sum(axis=1).T - X.diagonal()
            else:
                sum_ds = X.sum(axis=0)
        elif norm == 'l2':
            if _is_tri:
                sum_ds = ((X**2).sum(axis=0) +
                          (X**2).sum(axis=1).T -
                          (X**2).diagonal())
            else:
                sum_ds = (X**2).sum(axis=0)

        if SS is not None:
            raise NotImplementedError
        dbias = sum_ds.reshape((m, 1))
        if counts_profile is not None:
            dbias /= counts_profile[:, np.newaxis]
        # To avoid numerical instabilities
        dbias /= dbias[dbias != 0].mean()

        dbias[dbias == 0] = 1
        bias *= dbias

        if sparse.issparse(X):
            X = _update_normalization_csr(X, np.array(dbias).flatten())
        else:
            X /= dbias
            X /= dbias.T

        bias *= np.sqrt(X.sum() / total_counts)
        X *= total_counts / X.sum()

        if old_dbias is not None and np.abs(old_dbias - dbias).sum() < eps:
            if verbose > 1:
                print("break at iteration %d" % (it,))
            break

        if verbose > 1 and old_dbias is not None:
            print('ICE at iteration %d %s' %
                  (it, np.abs(old_dbias - dbias).sum()))

        old_dbias = dbias.copy()
    # Now that we are finished with the bias estimation, set all biases
    # corresponding to filtered rows to np.nan
    if sparse.issparse(X):
        X = X.tocoo()
        to_rm = (np.array(X.sum(axis=0)).flatten() +
                 np.array(X.sum(axis=1)).flatten()) == 0
    else:
        to_rm = (X.sum(axis=0) + X.sum(axis=1)) == 0
    bias[to_rm] = np.nan
    if output_bias:
        return X, bias
    else:
        return X


def SCN_normalization(X, max_iter=300, eps=1e-6, copy=True):
    """
    Sequential Component Normalization

    Parameters
    ----------
    X : ndarray (n, n)
        raw interaction frequency matrix

    max_iter : integer, optional, default: 300
        Maximum number of iteration

    eps : float, optional, default: 1e-6
        the relative increment in the results before declaring convergence.

    copy : boolean, optional, default: True
        If copy is True, the original data is not modified.

    Returns
    -------
    X : ndarray,
        Normalized IF
    """
    # X needs to be square, else it's gonna fail

    m, n = X.shape
    if m != n:
        raise ValueError

    if copy:
        X = X.copy()
    X = X.astype(float)

    for it in np.arange(max_iter):
        sum_X = np.sqrt((X ** 2).sum(0))
        sum_X[sum_X == 0] = 1
        X /= sum_X
        X = X.T
        sum_X = np.sqrt((X ** 2).sum(0))
        sum_X[sum_X == 0] = 1
        X /= sum_X
        X = X.T

        if np.abs(X - X.T).sum() < eps:
            print("break at iteration %d" % (it,))
            break

    return X



###############################################################################
#
# 160906 Giancarlo Bonora
# Output 3D co-ordinates in PDB format a la coords.cpp
#
###############################################################################

import os
import numpy as np
from scipy import sparse

from .config import parse
from .optimization import MDS, PM1, PM2, NMDS
from . import fastio
from .externals import iced
from .io import writePDB


max_iter = 5


###############################################################################

def run_mds(directory):
    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)

    random_state = np.random.RandomState(seed=options["seed"])

    # First, compute MDS
    if options["lengths"].endswith(".bed"):
        lengths = fastio.load_lengths(
            os.path.join(directory,
                         options["lengths"]))
    else:
        lengths = None

    if options["counts"].endswith("npy"):
        counts = np.load(os.path.join(directory, options["counts"]))
    elif options["counts"].endswith(".matrix"):
        counts = fastio.load_counts(
            os.path.join(directory,
                         options["counts"]),
            lengths=lengths)

    if options["normalize"]:
        counts = iced.filter.filter_low_counts(counts, sparsity=False,
                                               percentage=0.04)
        counts = iced.normalization.ICE_normalization(
            counts,
            max_iter=300)

    if not sparse.issparse(counts):
        counts = sparse.coo_matrix(counts)
    else:
        counts = counts.tocsr()
        counts.eliminate_zeros()
        counts = counts.tocoo()

    mds = MDS(alpha=options["alpha"],
              beta=options["beta"],
              random_state=random_state,
              max_iter=options["max_iter"],
              verbose=options["verbose"])
    X = mds.fit(counts)
    torm = np.array((counts.sum(axis=0) == 0)).flatten()
    X[torm] = np.nan
    np.savetxt(
        os.path.join(
            directory,
            "MDS." + options["output_name"]),
        X)

    # PDB file
    pdbfilename = os.path.join(
        directory,
        "MDS." + options["output_name"] + ".pdb")
    # pdbfilename = "test.pdb"
    writePDB(X, pdbfilename)

    return True


###############################################################################

def run_nmds(directory):
    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)

    random_state = np.random.RandomState(seed=options["seed"])

    # First, compute MDS
    if options["lengths"].endswith(".bed"):
        lengths = fastio.load_lengths(
            os.path.join(directory,
                         options["lengths"]))
    else:
        lengths = None

    if options["counts"].endswith("npy"):
        counts = np.load(os.path.join(directory, options["counts"]))
    elif options["counts"].endswith(".matrix"):
        counts = fastio.load_counts(
            os.path.join(directory,
                         options["counts"]),
            lengths=lengths)

    if options["normalize"]:
        counts = iced.filter.filter_low_counts(counts, sparsity=False,
                                               percentage=0.04)
        counts = iced.normalization.ICE_normalization(
            counts,
            max_iter=300)

    if not sparse.issparse(counts):
        counts = sparse.coo_matrix(counts)
    else:
        counts = counts.tocsr()
        counts.eliminate_zeros()
        counts = counts.tocoo()

    torm = np.array((counts.sum(axis=0) == 0)).flatten()
    nmds = NMDS(alpha=options["alpha"],
                beta=options["beta"],
                random_state=random_state,
                max_iter=options["max_iter"],
                verbose=options["verbose"])
    X = nmds.fit(counts)

    X[torm] = np.nan
    np.savetxt(
        os.path.join(
            directory,
            "NMDS." + options["output_name"]),
        X)

    # PDB file
    pdbfilename = os.path.join(
        directory,
        "NMDS." + options["output_name"] + ".pdb")
    # pdbfilename = "test.pdb"
    writePDB(X, pdbfilename)

    return True


###############################################################################

def run_pm1(directory):
    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)

    random_state = np.random.RandomState(seed=options["seed"])

    options = parse(config_file)

    if options["lengths"].endswith(".bed"):
        lengths = fastio.load_lengths(
            os.path.join(directory,
                         options["lengths"]))
    else:
        lengths = None

    if options["counts"].endswith("npy"):
        counts = np.load(os.path.join(directory, options["counts"]))
        counts[np.isnan(counts)] = 0
    elif options["counts"].endswith(".matrix"):
        counts = fastio.load_counts(
            os.path.join(directory,
                         options["counts"]),
            lengths=lengths)

    if options["normalize"]:
        counts = iced.filter.filter_low_counts(counts, sparsity=False,
                                               percentage=0.04)

        _, bias = iced.normalization.ICE_normalization(
            counts,
            max_iter=300,
            output_bias=True)
    else:
        bias = None

    if not sparse.issparse(counts):
        counts[np.isnan(counts)] = 0
        counts = sparse.coo_matrix(counts)
    else:
        counts = counts.tocsr()
        counts.eliminate_zeros()
        counts = counts.tocoo()

    pm1 = PM1(alpha=options["alpha"],
              beta=options["beta"],
              random_state=random_state,
              max_iter=options["max_iter"],
              bias=bias,
              verbose=options["verbose"])
    X = pm1.fit(counts)
    torm = np.array((counts.sum(axis=0) == 0)).flatten()

    X[torm] = np.nan

    np.savetxt(
        os.path.join(
            directory,
            "PM1." + options["output_name"]),
        X)

    # PDB file
    pdbfilename = os.path.join(
        directory,
        "PM1." + options["output_name"] + ".pdb")
    # pdbfilename = "test.pdb"
    writePDB(X, pdbfilename)

    return True


###############################################################################

def run_pm2(directory):
    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)

    random_state = np.random.RandomState(seed=options["seed"])

    options = parse(config_file)

    if options["lengths"].endswith(".bed"):
        lengths = fastio.load_lengths(
            os.path.join(directory,
                         options["lengths"]))
    else:
        lengths = None

    if options["counts"].endswith("npy"):
        counts = np.load(os.path.join(directory, options["counts"]))
        counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    elif options["counts"].endswith(".matrix"):
        counts = fastio.load_counts(
            os.path.join(directory, options["counts"]),
            lengths=lengths)

    if options["normalize"]:
        counts = iced.filter.filter_low_counts(counts, sparsity=False,
                                               percentage=0.04)

        _, bias = iced.normalization.ICE_normalization(
            counts,
            max_iter=300,
            output_bias=True)
    else:
        bias = None

    if not sparse.issparse(counts):
        counts[np.isnan(counts)] = 0
        counts = sparse.coo_matrix(counts)
    else:
        counts = counts.tocsr()
        counts.eliminate_zeros()
        counts = counts.tocoo()

    pm2 = PM2(alpha=options["alpha"],
              beta=options["beta"],
              random_state=random_state,
              max_iter=options["max_iter"],
              bias=bias,
              verbose=options["verbose"])
    X = pm2.fit(counts)

    torm = np.array((counts.sum(axis=0) == 0)).flatten()

    X[torm] = np.nan

    np.savetxt(
        os.path.join(
            directory,
            "PM2." + options["output_name"]),
        X) 
    # PDB file
    pdbfilename = os.path.join(
        directory,
        "PM2." + options["output_name"] + ".pdb")
    # pdbfilename = "test.pdb"
    writePDB(X, pdbfilename)

    return True



from nose.tools import assert_raises

import numpy as np

from pastis.utils import _check_squared_array


def test_check_squared_array():
    X = np.zeros((10, 9, 0))
    assert_raises(ValueError, _check_squared_array, X)

    X = np.zeros((10, 9))
    assert_raises(ValueError, _check_squared_array, X)



from .fastio_ import loadtxt, savetxt
try:
    from ._io_pandas import load_counts, load_lengths
    from ._io_pandas import write_counts
except ImportError:
    from ._io_else import load_lengths, load_counts
    from ._io_else import write_counts


def write_lengths(filename, lengths, resolution=1):
    """
    Write lengths as bed file
    """
    chromosomes = ["Chr%02d" % (i + 1) for i in range(len(lengths))]
    j = 0
    with open(filename, "w") as bed_file:
        for chrid, l in enumerate(lengths):
            for i in range(l):
                bed_file.write(
                    "%s\t%d\t%d\t%d\n" % (chromosomes[chrid],
                                          i * resolution + 1,
                                          (i + 1) * resolution,
                                          j))
                j += 1



import os
import numpy as np
from scipy import sparse
from .optimization import solver

from sklearn.metrics import euclidean_distances
from sklearn.isotonic import IsotonicRegression

from .config import parse


max_iter = 5

CMD_PM = ('%s -o %s -w 8 '
          '-r %d '
          '-k %s '
          '-i %s '
          '-c %s -y 1 -a %f -b %f > %s')


CMD_MDS = ('%s -o %s -w 8 '
           '-r %d '
           '-k %s '
           '-i %s '
           '-c %s -y 1 > %s')


def run_mds(directory):
    print "Running MDS"

    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)

    random_state = np.random.RandomState(seed=options["seed"])

    # XXX That should change !!
    counts = np.load(os.path.join(directory,
                                  options["counts"]))

    wish_distances = compute_wish_distances(counts, alpha=options["alpha"],
                                            beta=options["beta"])
    wish_distances[np.arange(len(counts)), np.arange(len(counts))] = 0
    wish_distances = sparse.coo_matrix(np.triu(wish_distances))

    X = 1. - 2. * random_state.rand(len(counts) * 3)
    lengths = np.array([wish_distances.shape[0]])

    mds = solver.MDS()
    X = mds.fit(wish_distances, lengths, init=X)
    # Save the results
    mask = counts.sum(axis=0) == 0
    X[mask] = np.nan
    np.savetxt(os.path.join(
        directory,
        "MDS." + options["output_name"]) + ".coord", X)

    return X


def compute_wish_distances(counts, alpha=-3, beta=1):
    """
    Computes wish distances from a counts matrix

    Parameters
    ----------
    counts : ndarray
        Interaction counts matrix

    alpha : float, optional, default: -3
        Coefficient of the power law

    beta : float, optional, default: 1
        Scaling factor

    Returns
    -------
    wish_distances
    """
    wish_distances = counts.copy() / beta
    wish_distances[wish_distances != 0] **= 1. / alpha
    # FIXME this doesn't use beta
    return beta * wish_distances


def run_nmds(directory):
    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)
    X = run_mds(directory)

    for i in range(0, max_iter):
        X[np.isnan(X)] = 0

        dis = euclidean_distances(X)
        counts = np.load(
            os.path.join(directory, options["counts"]))
        counts[np.isnan(counts)] = 0

        wish_distances = np.zeros(counts.shape)
        lengths = np.array([wish_distances.shape[0]])

        print "Fitting isotonic regression..."
        ir = IsotonicRegression()
        wish_distances[counts != 0] = ir.fit_transform(
            1. / counts[counts != 0],
            dis[counts != 0])
        wish_distances = sparse.coo_matrix(np.triu(wish_distances))
        mds = solver.MDS()
        X = mds.fit(wish_distances, lengths, init=X)
        # Save the results
        mask = counts.sum(axis=0) == 0
        X[mask] = np.nan
        np.savetxt(os.path.join(
            directory,
            "NMDS.%d." % (i, ) + options["output_name"]) + ".coord", X)

    np.savetxt(os.path.join(
        directory,
        "NMDS." + options["output_name"]) + ".coord", X)
    return X


def run_pm1(directory):
    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)
    X = run_mds(directory)

    counts = np.load(os.path.join(directory, options["counts"]))
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    counts = sparse.coo_matrix(np.triu(counts))

    lengths = np.array([counts.shape[0]])

    po = solver.PoissonOptimizer(max_iter=100000,
                                 alpha=options["alpha"],
                                 beta=options["beta"])

    X = po.fit(counts, lengths, init=X)
    np.savetxt(os.path.join(
        directory,
        "PM1." + options["output_name"]) + ".coord", X)
    return X


def run_pm2(directory):
    from .poisson_model_power_law import estimate_alpha_beta
    if os.path.exists(os.path.join(directory,
                                   "config.ini")):
        config_file = os.path.join(directory, "config.ini")
    else:
        config_file = None

    options = parse(config_file)
    alpha, beta = options["alpha"], options["beta"]
    X = run_mds(directory)

    # Save counts in a format the C++ code can use
    counts = np.load(
        os.path.join(directory, options["counts"]))
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0

    counts = sparse.coo_matrix(np.triu(counts))
    lengths = np.array([counts.shape[0]])

    for i in range(0, max_iter):
        X[np.isnan(X)] = 0
        # Fit alpha and beta
        if i != 0:
            alpha, beta = estimate_alpha_beta(
                counts.toarray() + counts.toarray().T,
                X, ini=np.array([alpha]))

        po = solver.PoissonOptimizer(max_iter=100000, alpha=alpha, beta=beta)
        X = po.fit(counts, lengths, init=X)

        # Save the results
        mask = counts.sum(axis=0) == 0
        X[mask] = np.nan
        np.savetxt(os.path.join(
            directory,
            "PM2.%d." % (i, ) + options["output_name"]) + ".coord", X)

    np.savetxt(os.path.join(
        directory,
        "PM2." + options["output_name"]) + ".coord", X)
    return X



import numpy as np
from scipy import optimize
from scipy import sparse
from sklearn.utils import check_random_state
from sklearn.metrics import euclidean_distances
from sklearn.isotonic import IsotonicRegression

from .utils import compute_wish_distances


def MDS_obj(X, distances):
    if sparse.issparse(distances):
        return MDS_obj_sparse(X, distances)
    else:
        return MDS_obj_dense(X, distances)


def MDS_obj_dense(X, distances):
    X = X.reshape(-1, 3)
    dis = euclidean_distances(X)
    X = X.flatten()
    obj = 1. / distances ** 2 * (dis - distances) ** 2
    return obj[np.invert(np.isnan(obj) | np.isinf(obj))].sum()


def MDS_obj_sparse(X, distances):
    X = X.reshape(-1, 3)
    dis = np.sqrt(((X[distances.row] - X[distances.col])**2).sum(axis=1))
    return ((dis - distances.data)**2 / distances.data**2).sum()


def MDS_gradient(X, distances):
    if sparse.issparse(distances):
        return MDS_gradient_sparse(X, distances)
    else:
        return MDS_gradient_dense(X, distances)


def MDS_gradient_dense(X, distances):
    X = X.reshape(-1, 3)
    m, n = X.shape
    tmp = X.repeat(m, axis=0).reshape((m, m, n))
    dif = tmp - tmp.transpose(1, 0, 2)
    dis = euclidean_distances(X).repeat(3, axis=1).flatten()
    distances = distances.repeat(3, axis=1).flatten()
    grad = 2 * dif.flatten() * (dis - distances) / dis / distances**2
    grad[(distances == 0) | np.isnan(grad)] = 0
    X = X.flatten()
    return grad.reshape((m, m, n)).sum(axis=1).flatten()


def MDS_gradient_sparse(X, distances):
    X = X.reshape(-1, 3)
    dis = np.sqrt(((X[distances.row] - X[distances.col])**2).sum(axis=1))

    grad = 2 * ((dis - distances.data) / dis /
                distances.data**2)[:, np.newaxis] * (
        X[distances.row] - X[distances.col])
    grad_ = np.zeros(X.shape)

    for i in range(X.shape[0]):
        grad_[i] += grad[distances.row == i].sum(axis=0)
        grad_[i] -= grad[distances.col == i].sum(axis=0)

    X = X.flatten()
    return grad_.flatten()


def estimate_X(counts, alpha=-3., beta=1., ini=None,
               verbose=0,
               use_zero_entries=False,
               precompute_distances=False,
               bias=None,
               random_state=None, type="MDS2",
               factr=1e12,
               maxiter=10000):
    n = counts.shape[0]

    random_state = check_random_state(random_state)
    if ini is None or ini == "random":
        ini = 1 - 2 * random_state.rand(n * 3)
    if not precompute_distances or precompute_distances == "auto":
        distances = compute_wish_distances(counts, alpha=alpha, beta=beta,
                                           bias=bias)
    else:
        if bias is not None:
            counts /= bias
            counts /= bias.T
        distances = counts
    results = optimize.fmin_l_bfgs_b(
        MDS_obj, ini.flatten(),
        MDS_gradient,
        (distances, ),
        iprint=1,
        factr=factr,
        maxiter=maxiter)
    return results[0].reshape(-1, 3)


class MDS(object):
    """
    """
    def __init__(self, alpha=-3., beta=1.,
                 max_iter=5000, random_state=None, n_init=1, n_jobs=1,
                 precompute_distances="auto", bias=None,
                 init=None, verbose=False, factr=1e12):
        self.max_iter = max_iter
        self.alpha = alpha
        self.beta = beta
        self.max_iter = max_iter
        self.random_state = check_random_state(random_state)
        self.precompute_distances = precompute_distances
        self.n_init = n_init
        self.n_jobs = n_jobs
        self.init = init
        self.verbose = verbose
        self.bias = bias
        self.factr = factr

    def fit(self, counts, lengths=None):
        """

        """
        if not sparse.isspmatrix_coo(counts):
            counts = sparse.coo_matrix(counts)

        X_ = estimate_X(counts,
                        alpha=self.alpha,
                        beta=self.beta,
                        ini=self.init,
                        verbose=self.verbose,
                        precompute_distances=self.precompute_distances,
                        use_zero_entries=False,
                        random_state=self.random_state,
                        bias=self.bias,
                        factr=self.factr,
                        maxiter=self.max_iter)
        return X_


class NMDS(object):
    """
    """
    def __init__(self, alpha=-3., beta=1.,
                 max_iter=5000, random_state=None, n_init=1, n_jobs=1,
                 precompute_distances="auto", bias=None,
                 init=None, verbose=False, max_iter_outer=5, factr=1e12):
        self.max_iter = max_iter
        self.alpha = alpha
        self.beta = beta
        self.max_iter = max_iter
        self.random_state = check_random_state(random_state)
        self.precompute_distances = precompute_distances
        self.n_init = n_init
        self.n_jobs = n_jobs
        self.init = init
        self.verbose = verbose
        self.bias = bias
        self.factr = factr
        self.max_iter_outer = max_iter_outer

    def fit(self, counts, lengths=None):
        """

        """
        if not sparse.isspmatrix_coo(counts):
            counts = sparse.coo_matrix(counts)

        for i in range(self.max_iter_outer):
            if i == 0:
                X = estimate_X(counts,
                               alpha=self.alpha,
                               beta=self.beta,
                               ini=self.init,
                               verbose=self.verbose,
                               use_zero_entries=False,
                               random_state=self.random_state,
                               bias=self.bias,
                               factr=self.factr,
                               maxiter=self.max_iter)
            else:
                ir = IsotonicRegression()
                dis = np.sqrt(
                    ((X[counts.row] - X[counts.col])**2).sum(axis=1))
                wish_distances = ir.fit_transform(
                    1. / counts.data,
                    dis)
                X = estimate_X(sparse.coo_matrix((wish_distances,
                                                  (counts.row,
                                                   counts.col))),
                               alpha=self.alpha,
                               beta=self.beta,
                               ini=X,
                               verbose=self.verbose,
                               use_zero_entries=False,
                               precompute_distances="precomputed",
                               random_state=self.random_state,
                               factr=self.factr,
                               maxiter=self.max_iter)
        print("writing wish distances")
        return X



import ConfigParser
import os


def get_default_options():
    """
    Returns default options

    Here are all the options:

    output_name : structure.pdb, str
        The name of the PDB file to write. The name of the algorithm used will
        be appended to this filename. The coordinates of each beads will also
        be outputted in a text file `structure.pdb.txt`.
        For example, running the algorithm MDS with an output_name
        `structure.pdb` will yield the files `MDS.structure.pdb` and
        `MDS.structure.pdb.txt`

    counts : data/counts.npy, str
        The numpy ndarray of counts

    organism_structure : files/budding_yeast_structure
        The path to the filename containing the organism structure. This file
        should contain the lengths of the chromosomes of the organism.

    resolution : 10000, integer
        Resolution to which perform the optimization.

    chromosomes : "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16", str
        coma separated list of chromosomes onto which the optimization should
        be performed.

    binary_mds : "MDS_all"
        Path to the binary MDS_all

    binary_pm : "PM_all"
        Path to the binary PM_all

    alpha : -3., float
        Coefficient of the power law used in converting interaction counts to
        wish distances

    beta : 1., float
        Scaling factor of the structure.

    nucleus_size : float, optional, default: None
        The size of the nucleus. If None, no constraints will be applied.
        New in 0.1

    adjacent_beads : float, optional, default: None
        The distances between adjacent beads. If None, no constraints will be
        applied.
        New in 0.1

    seed : 0, integer
        Random seed used when generating the starting point in the
        optimization.
    """
    options = {"output_name": "structure.pdb",
               "resolution": 10000,
               "input_name": "wish_distances.txt",
               "counts": "data/counts.npy",
               "chromosomes": "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16",
               "organism_structure": "files/budding_yeast_structure",
               "alpha": -3.,
               "beta": 1.,
               "logging_file": "MDS.log",
               "binary_mds": "MDS_all",
               "binary_pm": "PM_all",
               "nucleus_size": None,
               "adjacent_beads": None,
               "seed": 0,
               }

    return options


def parse(filename=None):
    """
    Parses a configuration file.

    Parameters
    ----------
    filename : str, optional, default: None
        If a filename is provided, reads the configuration file, and returns
        the options. If None, returns the default options.

    Returns
    -------
    options : dict
    """
    options = get_default_options()
    if filename is None:
        return options

    if not os.path.exists(filename):
        raise IOError("File %s doesn't existe" % filename)

    config = ConfigParser.ConfigParser()
    config.readfp(open(filename))
    for key in options.iterkeys():
        try:
            options[key] = type(options[key])(
                config.get("all", key))
        except ConfigParser.NoOptionError:
            pass
    return options



import numpy as np
from nose.tools import assert_equal
from numpy.testing import assert_array_equal

from iced.utils._genome import get_intra_mask
from iced.utils._genome import get_inter_mask
from iced.utils._genome import _change_lengths_resolution
from iced.utils._genome import undersample_per_chr
from iced.utils._genome import extract_sub_contact_map
from iced.utils._genome import downsample_resolution


def test_get_intra_mask():
    lengths = np.array([5, 5])
    mask = get_intra_mask(lengths)
    true_mask = np.zeros((10, 10))
    true_mask[:5, :5] = 1
    true_mask[5:, 5:] = 1
    assert_array_equal(mask, true_mask.astype(bool))


def test_change_lengths_resolution():
    lengths = np.array([5, 5])
    l = _change_lengths_resolution(lengths, resolution=1)
    assert_array_equal(lengths, l)


def test_get_inter_mask():
    lengths = np.array([5, 5])
    mask = get_inter_mask(lengths)
    true_mask = np.zeros((10, 10))
    true_mask[:5, :5] = 1
    true_mask[5:, 5:] = 1
    assert_array_equal(mask, np.invert(true_mask.astype(bool)))


def test_downsample_resolution():
    random_state = np.random.RandomState(seed=42)

    lengths = np.array([10, 10])
    counts = np.triu(random_state.randint(
        0, 100, (lengths.sum(), lengths.sum())))
    counts = counts + counts.T
    downsampled_counts, downsampled_lengths = downsample_resolution(
        counts, lengths)
    assert_equal(downsampled_lengths.sum(), lengths.sum()/2)

    lengths = np.array([10, 11])
    counts = np.triu(random_state.randint(
        0, 100, (lengths.sum(), lengths.sum())))
    counts = counts + counts.T
    downsampled_counts, downsampled_lengths = downsample_resolution(
        counts, lengths)
    assert_equal(downsampled_lengths.sum(), 11)


def test_undersample_per_chr():
    X = np.array([[1, 1, 0, 0],
                  [1, 1, 0, 0],
                  [0, 0, 0.5, 0.5],
                  [0, 0, 0.5, 0.5]])
    lengths = np.array([2, 2])
    undersampled_X = undersample_per_chr(X, lengths)
    undersampled_X_true = np.array([[1, 0],
                                    [0, 0.5]])
    assert_array_equal(undersampled_X_true, undersampled_X)


def test_return_sample():
    lengths = np.array([50, 75])
    n = lengths.sum()
    X = np.random.randint(0, 50, (n, n))
    X = np.triu(X)
    sub_X, _ = extract_sub_contact_map(X, lengths, [0])
    assert_array_equal(X[:lengths[0], :lengths[0]],
                       sub_X)



import numpy as np
import numbers


def _check_squared_array(X):
    """
    Check whether arrays are squared

    Parameters
    ----------
    X : ndarray

    Returns
    -------
    X
    """
    if len(X.shape) != 2:
        raise ValueError(
            "The ndarray has %d dimension. 2D array is expected." %
            len(X.shape))

    if X.shape[0] != X.shape[1]:
        raise ValueError(
            "The ndarray is of shape (%d, %d). Squasev array is expected." %
            (X.shape[0], X.shape[1]))

    return X


def check_random_state(seed):
    """Turn seed into a np.random.RandomState instance

    If seed is None, return the RandomState singleton used by np.random.
    If seed is an int, return a new RandomState instance seeded with seed.
    If seed is already a RandomState instance, return it.
    Otherwise raise ValueError.
    """
    if seed is None or seed is np.random:
        return np.random.mtrand._rand
    if isinstance(seed, (numbers.Integral, np.integer)):
        return np.random.RandomState(seed)
    if isinstance(seed, np.random.RandomState):
        return seed
    raise ValueError('%r cannot be used to seed a numpy.random.RandomState'
                     ' instance' % seed)



#! /usr/bin/env python

import sys
import argparse
import numpy as np
from scipy import sparse

import iced
from iced.io import loadtxt, savetxt


parser = argparse.ArgumentParser("ICE normalization")
parser.add_argument('filename',
                    metavar='File to load',
                    type=str,
                    help='Path to file of contact counts to load')
parser.add_argument("--results_filename",
                    "-r",
                    type=str,
                    default=None,
                    help="results_filename")
parser.add_argument("--filtering_perc", "-f",
                    type=float,
                    default=None,
                    help="Percentage of reads to filter out")
parser.add_argument("--filter_low_counts_perc",
                    type=float,
                    default=0.02,
                    help="Percentage of reads to filter out")
parser.add_argument("--filter_high_counts_perc",
                    type=float,
                    default=0,
                    help="Percentage of reads to filter out")
parser.add_argument("--max_iter", "-m", default=100, type=int,
                    help="Maximum number of iterations")
parser.add_argument("--eps", "-e", default=0.1, type=float,
                    help="Precision")
parser.add_argument("--dense", "-d", default=False, action="store_true")
parser.add_argument("--verbose", "-v", default=False)


args = parser.parse_args()
filename = args.filename

# Deprecating filtering_perc option
filter_low_counts = None
if "--filtering_perc" in sys.argv:
    DeprecationWarning(
        "Option '--filtering_perc' is deprecated. Please use "
        "'--filter_low_counts_perc' instead.'")
    # And print it again because deprecation warnings are not displayed for
    # recent versions of python
    print "--filtering_perc is deprecated. Please use filter_low_counts_perc"
    print "instead. This option will be removed in ice 0.3"
    filter_low_counts = args.filtering_perc
if "--filter_low_counts_perc" in sys.argv and "--filtering_perc" in sys.argv:
    raise Warning("This two options are incompatible")
if "--filtering_perc" is None and "--filter_low_counts_perc" not in sys.argv:
    filter_low_counts_perc = 0.02
elif args.filter_low_counts_perc is not None:
    filter_low_counts_perc = args.filter_low_counts_perc

if args.verbose:
    print "Loading files..."

# Loads file as i, j, counts
i, j, data = loadtxt(filename).T
N = max(i.max(), j.max()) + 1
counts = sparse.coo_matrix((data, (i, j)), shape=(N, N), dtype=float)
if args.dense:
    counts = np.array(counts.todense())
else:
    counts = sparse.csr_matrix(counts)

if args.verbose:
    print "Normalizing..."

if filter_low_counts_perc != 0:
    print filter_low_counts_perc
    counts = iced.filter.filter_low_counts(counts,
                                           percentage=filter_low_counts_perc,
                                           copy=False, sparsity=False)
if args.filter_high_counts_perc != 0:
    counts = iced.filter.filter_high_counts(
        counts,
        percentage=args.filter_high_counts_perc,
        copy=False)

counts = iced.normalization.ICE_normalization(
    counts, max_iter=args.max_iter, copy=False,
    verbose=args.verbose, eps=args.eps)

if args.results_filename is None:
    results_filename = ".".join(
        filename.split(".")[:-1]) + "_normalized." + filename.split(".")[-1]
else:
    results_filename = args.results_filename

counts = sparse.coo_matrix(counts)

if args.verbose:
    print "Writing results..."
savetxt(results_filename, counts.col, counts.row, counts.data)



import numpy as np
from sklearn.metrics import euclidean_distances


def poisson_model_obj(X, counts, beta=1., alpha=-3, use_zero_counts=False):
    dis = euclidean_distances(X)
    log_likelihood = counts * alpha * np.log(dis) - beta * (dis ** alpha)
    if use_zero_counts:
        val = log_likelihood[
            np.invert(np.isnan(log_likelihood))].sum()
    else:
        val = log_likelihood[
            np.invert(np.isnan(log_likelihood) | (counts == 0))].sum()
    return -val


def negative_binomial_obj(X, counts, beta=1., alpha=-3., d=7.,
                          use_zero_counts=False):
    dis = euclidean_distances(X)
    log_likelihood = (counts * alpha * np.log(dis) -
                      (counts + d) * np.log(d + beta * (dis ** alpha)))
    if use_zero_counts:
        val = -log_likelihood[
            np.invert(np.isnan(log_likelihood))].sum()
    else:
        val = -log_likelihood[
            np.invert(np.isnan(log_likelihood) | (counts == 0))].sum()
    return val



import os
from os.path import join
import warnings


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration
    from numpy.distutils.system_info import get_info, BlasNotFoundError
    import numpy

    libraries = []
    if os.name == 'posix':
        libraries.append('m')

    config = Configuration('pastis', parent_package, top_path)
    config.add_subpackage('utils')
    config.add_subpackage("optimization")
    config.add_subpackage("externals")
    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())




import numpy as np
from scipy import sparse

from .optimization_ import Optimizer


class PoissonOptimizer(object):
    """
    """
    def __init__(self):
        self.optimizer = Optimizer(model="poisson")

    def fit(self, counts, lengths, init=None, random_state=None):
        c = sparse.coo_matrix(counts)
        row, col, data = c.row.astype(int), c.col.astype(int), c.data
        if init is None:
            init = np.random.randn((lengths.sum() * 3))
        else:
            init = init.flatten()
        self.init = init
        self.X_ = self.optimizer.fit(
            row, col, data, counts.shape[0], lengths,
            init, dispersion=init)
        return self.X_


class NegativeBinomialOptimizer(object):
    """
    """
    def __init__(self):
        self.optimizer = Optimizer(model="negative_binomial")

    def fit(self, counts, lengths, dispersion, init=None, random_state=None):
        c = sparse.coo_matrix(counts)
        row, col, data = c.row.astype(int), c.col.astype(int), c.data
        if init is None:
            init = np.random.randn((lengths.sum() * 3))
        else:
            init = init.flatten()
        self.init = init

        if len(dispersion) != len(data):
            raise ValueError("Disperson should be of same length as data")
        self.X_ = self.optimizer.fit(
            row, col, data, counts.shape[0], lengths,
            init, dispersion=dispersion.astype(float))
        return self.X_


class MDS(object):
    """
    """
    def __init__(self):
        self.optimizer = Optimizer(model="mds")

    def fit(self, counts, lengths, init=None, random_state=None):
        c = sparse.coo_matrix(counts)
        row, col, data = c.row.astype(int), c.col.astype(int), c.data
        if init is None:
            init = np.random.randn((lengths.sum() * 3))
        else:
            init = init.flatten()
        self.init_ = init.copy()
        self.X_ = self.optimizer.fit(
            row, col, data, counts.shape[0], lengths,
            init, dispersion=init)
        return self.X_



import numpy as np
from scipy import sparse
from sklearn.utils import check_random_state

from .optimization_ import Optimizer


class PoissonOptimizer(object):
    """
    """
    def __init__(self, max_iter=5000, hessian_approximation=True, tol=1e-1,
                 acceptable_tol=1e-1, mu_strategy="adaptive", nucleus_size=-1,
                 adjacent_beads=-1, alpha=-3, beta=1):

        if mu_strategy == "monotone":
            mu_strategy_type = 0
        elif mu_strategy == "adaptive":
            mu_strategy_type = 1
        else:
            raise ValueError(
                "Unkown mu strategy type")

        self.optimizer = Optimizer(
            model="poisson",
            max_iter=max_iter,
            hessian_approximation=hessian_approximation,
            tol=tol,
            acceptable_tol=acceptable_tol,
            mu_strategy=mu_strategy_type,
            nucleus_size=nucleus_size,
            adjacent_beads=adjacent_beads,
            alpha=alpha,
            beta=beta)

    def fit(self, counts, lengths, init=None, random_state=None, mapping=None,
            bias=None):
        """
        Fitting the poisson model.
        """

        if mapping is not None and (mapping.max() + 1 > len(counts)):
            raise ValueError(
                "Elements from the mapping are outside of the contact "
                "counts'range.")
        if mapping is not None and len(mapping) != lengths.sum():
            raise ValueError(
                "The mapping should be of shape L, where L is the total "
                "lengths of the genome")

        c = sparse.coo_matrix(counts)
        row, col, data = c.row.astype(int), c.col.astype(int), c.data
        random_state = check_random_state(random_state)

        if bias is None:
            bias = np.ones(lengths.sum(), dtype=np.double)
        if init is None:
            init = random_state.randn((lengths.sum() * 3))
        else:
            init = init.flatten()
        self.init = init

        if mapping is None:
            # This is the haploid case. We need to create the mapping by hand.
            mapping_data = np.arange(lengths.sum())
            mapping_num_elements = np.ones(mapping_data.shape)
        else:
            mapping_data = []
            mapping_num_elements = []
            for i in np.unique(mapping):
                mapping_data.append(np.where(mapping == i)[0])
                mapping_num_elements.append(len(np.where(mapping == i)[0]))
            mapping_data = np.concatenate(mapping_data)
            mapping_num_elements = np.array(mapping_num_elements)

        mapping_indptr = np.concatenate(
            [[0],
             mapping_num_elements.cumsum()[:-1]]).astype(np.int32)

        self.X_ = self.optimizer.fit_(
            row, col, data, counts.shape[0], lengths,
            init,
            bias=bias,
            mapping_data=mapping_data.astype(np.int32),
            mapping_num_elements=mapping_num_elements.astype(np.int32),
            mapping_indptr=mapping_indptr.astype(np.int32))

        return self.X_


class NegativeBinomialOptimizer(object):
    """
    """
    def __init__(self, max_iter=5000, hessian_approximation=True, tol=1e-1,
                 acceptable_tol=1e-1,  mu_strategy="adaptive", alpha=-3,
                 beta=1):

        if mu_strategy == "monotone":
            mu_strategy_type = 0
        elif mu_strategy == "adaptive":
            mu_strategy_type = 1
        else:
            raise ValueError(
                "Unkown mu strategy type")

        self.optimizer = Optimizer(
            model="negative_binomial",
            max_iter=max_iter,
            hessian_approximation=hessian_approximation,
            tol=tol,
            acceptable_tol=acceptable_tol,
            mu_strategy=mu_strategy_type,
            alpha=alpha,
            beta=beta)

    def fit(self, counts, lengths, dispersion, init=None, random_state=None,
            bias=None):
        c = sparse.coo_matrix(counts)
        row, col, data = c.row.astype(int), c.col.astype(int), c.data
        random_state = check_random_state(random_state)

        if bias is None:
            bias = np.ones(lengths.sum(), dtype=np.double)

        if init is None:
            init = random_state.randn((lengths.sum() * 3))
        else:
            init = init.flatten()
        self.init = init

        if len(dispersion) != len(data):
            raise ValueError("Disperson should be of same length as data")
        self.X_ = self.optimizer.fit(
            row, col, data, counts.shape[0], lengths,
            init, dispersion=dispersion.astype(float),
            bias=bias)

        return self.X_


class MDS(object):
    """
    """
    def __init__(self, max_iter=5000, hessian_approximation=True, tol=1e-1,
                 acceptable_tol=1e-1, mu_strategy="adaptive", nucleus_size=-1,
                 adjacent_beads=-1):
        if mu_strategy == "monotone":
            mu_strategy_type = 0
        elif mu_strategy == "adaptive":
            mu_strategy_type = 1
        else:
            raise ValueError(
                "Unkown mu strategy type")

        self.optimizer = Optimizer(
            model="mds",
            max_iter=max_iter,
            hessian_approximation=hessian_approximation,
            tol=tol,
            acceptable_tol=acceptable_tol,
            mu_strategy=mu_strategy_type,
            nucleus_size=nucleus_size,
            adjacent_beads=adjacent_beads)

    def fit(self, counts, lengths, init=None, random_state=None):
        """

        """
        # XXX This is not using the random state
        c = sparse.coo_matrix(counts)
        row, col, data = c.row.astype(int), c.col.astype(int), c.data
        random_state = check_random_state(random_state)
        if init is None:
            init = random_state.randn((lengths.sum() * 3))
        else:
            init = init.flatten()
        self.init_ = init.copy()
        self.X_ = self.optimizer.fit(
            row, col, data, counts.shape[0], lengths,
            init, dispersion=init, bias=init)
        return self.X_



import os
import shutil
from numpy.distutils.core import setup
from distutils.command.clean import clean as Clean


DISTNAME = 'pastis'
DESCRIPTION = 'A set of algorithms for the 3D inference of the genome'
MAINTAINER = 'Nelle Varoquaux'
MAINTAINER_EMAIL = 'nelle.varoquaux@ensmp.fr'
VERSION = '0.2.1'
LICENSE = "New BSD"
URL = 'http://cbio.ensmp.fr/pastis'
DOWNLOAD_URL = 'https://github.com/hiclib/pastis/releases'


class CleanCommand(Clean):
    description = ("Remove build directories, and compiled file in the "
                   "source tree")

    def run(self):
        Clean.run(self)
        if os.path.exists('build'):
            shutil.rmtree('build')
        for dirpath, dirnames, filenames in os.walk('pastis'):
            for filename in filenames:
                if (filename.endswith('.so') or filename.endswith('.pyd')
                   or filename.endswith('.dll')
                   or filename.endswith('.pyc')):
                    os.unlink(os.path.join(dirpath, filename))


def configuration(parent_package='', top_path=None):
    if os.path.exists('MANIFEST'):
        os.remove('MANIFEST')

    from numpy.distutils.misc_util import Configuration
    config = Configuration(None, parent_package, top_path)

    config.add_subpackage('pastis')

    return config


if __name__ == "__main__":
    setup(configuration=configuration,
          name=DISTNAME,
          maintainer=MAINTAINER,
          include_package_data=True,
          maintainer_email=MAINTAINER_EMAIL,
          description=DESCRIPTION,
          version=VERSION,
          url=URL,
          download_url=DOWNLOAD_URL,
          zip_safe=False,  # the package can run out of an .egg file
          scripts=["pastis/script/pastis-mds",
                   'pastis/script/pastis-nmds', "pastis/script/pastis-pm1",
                   "pastis/script/pastis-pm2"],
          classifiers=[
              'Intended Audience :: Science/Research',
              'Intended Audience :: Developers',
              'License :: OSI Approved',
              'Programming Language :: C',
              'Programming Language :: C++',
              'Programming Language :: Python',
              'Topic :: Software Development',
              'Topic :: Scientific/Engineering',
              'Operating System :: POSIX',
              'Operating System :: Unix',
              'Operating System :: MacOS'],
          install_requires=["numpy", "scipy", "scikit-learn", "argparse"],
          cmdclass={'clean': CleanCommand})



from os.path import join
import os
import numpy as np


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    config = Configuration('optimization', parent_package, top_path)

    # Counts module
    # XXX is this useful ?
    counts_sources = ['counts_.cpp',
                      join("counts.cpp"),
                      ]
    counts_depends = [join(".", 'counts.hpp')]

    config.add_extension('counts_',
                         sources=counts_sources,
                         include_dirs=[np.get_include(),
                                       os.path.expanduser("~/include/coin"),
                                       ],
                         depends=counts_depends,
                         libraries=["blas", "lapack", "gfortran",
                                    "pthread", "gfortranbegin", 'm', "gcc_s",
                                    "dl",
                                    ],
                         library_dirs=[os.path.expanduser('~/.local/lib')],
                         language='c++'
                         )
    optimization_sources = ['counts_.cpp',
                            join("counts.cpp"),
                            "optimization.cpp", "optimization_.cpp",
                            "poisson_model.cpp",
                            "model.cpp",
                            "mds.cpp",
                            "negative_binomial_model.cpp",
                            ]
    optimization_depends = [join(".", 'counts.hpp'),
                            "model.hpp",
                            "mds.hpp",
                            join(".", "optimization.hpp"),
                            join(".", "poisson_model.hpp"),
                            join(".", "negative_binomial_model.hpp"),]

    config.add_extension('optimization_',
                         sources=optimization_sources,
                         include_dirs=[np.get_include(),
                                       os.path.expanduser(
                                       "~/.local/include/coin"),
                                       ],
                         depends=optimization_depends,
                         libraries=["ipopt",
                                    "blas", "lapack", "gfortran",
                                    "pthread", "gfortranbegin", 'm', "gcc_s",
                                    "dl",
                                    "coinhsl"
                                    ],
                         library_dirs=[os.path.expanduser('~/.local/lib')],
                         language='c++'
                         )

    return config


if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



import numpy as np
from scipy import sparse
from sklearn.metrics import euclidean_distances
from numpy.testing import assert_array_almost_equal
from pastis.optimization import mds


def estimate_X_test():
    n = 50
    random_state = np.random.RandomState(42)
    X_true = random_state.rand(n, 3)
    dis = euclidean_distances(X_true)
    alpha, beta = -3., 1.

    counts = beta * dis ** alpha
    counts = np.triu(counts)
    counts[np.arange(len(counts)), np.arange(len(counts))] = 0
    counts = sparse.coo_matrix(counts)

    X = mds.estimate_X(counts, random_state=random_state)
    assert_array_almost_equal(dis,
                              euclidean_distances(X), 2)



import numpy as np
from scipy import sparse
from scipy import optimize
from sklearn.utils import check_random_state
from . import poisson_model
from . import mds


def poisson_obj(X, counts, alpha=-3., beta=1., bias=None,
                use_zero_counts=False, cst=0):
    if bias is None:
        bias = np.ones((counts.shape[0], 1))

    if sparse.issparse(counts):
        return _poisson_obj_sparse(
            X, counts, beta=beta, bias=bias,
            alpha=alpha,
            use_zero_counts=use_zero_counts, cst=cst)
    else:
        raise NotImplementedError(
            "Poisson model is not implemented for dense")


def _poisson_obj_sparse(X, counts, alpha=-3., beta=1., bias=None,
                        use_zero_counts=False, cst=0):

    if bias is None:
        bias = np.ones((X.shape[0], ))
    bias = bias.flatten()
    dis = np.sqrt(((X[counts.row] - X[counts.col])**2).sum(axis=1))
    fdis = bias[counts.row] * bias[counts.col] * beta * dis ** alpha

    obj = fdis.sum() - (counts.data * np.log(fdis)).sum()
    return obj


def poisson_gradient(X, counts, alpha=-3, beta=1, bias=None,
                     use_zero_counts=False):
    if bias is None:
        bias = np.ones((counts.shape[0], 1))

    if sparse.issparse(counts):
        return _poisson_gradient_sparse(
            X, counts, beta=beta, bias=bias,
            alpha=alpha)
    else:
        raise NotImplementedError(
            "Poisson model is not implemented for dense")


def _poisson_gradient_sparse(X, counts, alpha=-3, beta=1, bias=None):
    if bias is None:
        bias = np.ones((counts.shape[0], 1))

    bias = bias.flatten()
    dis = np.sqrt(((X[counts.row] - X[counts.col])**2).sum(axis=1))
    fdis = bias[counts.row] * bias[counts.col] * beta * dis ** alpha

    diff = X[counts.row] - X[counts.col]

    grad = - ((counts.data / fdis - 1) * fdis * alpha /
              (dis ** 2))[:, np.newaxis] * diff

    grad_ = np.zeros(X.shape)

    for i in range(X.shape[0]):
        grad_[i] += grad[counts.row == i].sum(axis=0)
        grad_[i] -= grad[counts.col == i].sum(axis=0)

    return grad_


def eval_f(x, user_data=None):
    n, counts, alpha, beta, bias, use_zero_counts = user_data
    x = x.reshape((n, 3))
    obj = poisson_obj(x, counts, alpha=alpha, beta=beta, bias=bias)
    x = x.flatten()
    return obj


def eval_grad_f(x, user_data=None):
    n, counts, alpha, beta, bias, use_zero_counts = user_data
    x = x.reshape((n, 3))
    grad = poisson_gradient(x, counts, alpha=alpha,
                            beta=beta, bias=bias)
    x = x.flatten()
    return grad.flatten()


def estimate_X(counts, alpha=-3., beta=1.,
               ini=None, bias=None,
               random_state=None, maxiter=10000, verbose=0):
    """
    Estimate the parameters of g

    Parameters
    ----------
    counts : sparse scipy matrix (n, n)

    alpha : float, optional, default: -3
        counts-to-distances mapping coefficient

    beta : float, optional, default: 1
        counts-to-distnances scaling coefficient

    init : ndarray (n, 3), optional, default: None
        initialization point

    bias : ndarray (n, 1), optional, default: None
        bias vector. If None, no bias will be applied to the model

    random_state : {RandomState, int, None}, optional, default: None
        random state object, or seed, or None.

    maxiter : int, optional, default: 10000
        Maximum number of iteration

    verbose : int, optional, default: 0
        verbosity

    Returns
    ------
    X : 3D structure

    """
    n = counts.shape[0]

    if not sparse.isspmatrix_coo(counts):
        counts = sparse.coo_matrix(counts)

    random_state = check_random_state(random_state)
    if ini is None:
        ini = 1 - 2 * random_state.rand(n * 3)

    data = (n, counts, alpha, beta, bias,
            False)

    results = optimize.fmin_l_bfgs_b(
        eval_f,
        ini.flatten(),
        eval_grad_f,
        (data, ),
        iprint=verbose,
        maxiter=maxiter,
        )
    results = results[0].reshape(-1, 3)
    return results


class PM1(object):
    """
    """
    def __init__(self, alpha=-3., beta=1.,
                 max_iter=5000, random_state=None, n_init=1, n_jobs=1,
                 init="MDS2", verbose=False, bias=None):
        self.max_iter = max_iter
        self.alpha = alpha
        self.beta = beta
        self.random_state = check_random_state(random_state)
        self.n_init = n_init
        self.bias = bias
        self.n_jobs = n_jobs
        self.init = init
        self.verbose = verbose

    def fit(self, counts, lengths=None):
        """

        """
        if not sparse.isspmatrix_coo(counts):
            counts = sparse.coo_matrix(counts)

        if self.init == "MDS2":
            if self.verbose:
                print "Initialing with MDS2"
            X = mds.estimate_X(counts, alpha=self.alpha,
                               beta=self.beta,
                               ini="random",
                               bias=self.bias,
                               random_state=self.random_state,
                               maxiter=self.max_iter,
                               verbose=self.verbose)
        else:
            X = self.init

        X = estimate_X(counts,
                       alpha=self.alpha,
                       beta=self.beta,
                       ini=X,
                       bias=self.bias,
                       verbose=self.verbose,
                       random_state=self.random_state,
                       maxiter=self.max_iter)
        return X


class PM2(object):
    """
    """
    def __init__(self, alpha=-3., beta=1.,
                 max_iter=5000, max_iter_outer_loop=5,
                 random_state=None, n_init=1, n_jobs=1,
                 bias=None,
                 init="MDS2", verbose=False):
        self.max_iter = max_iter
        self.alpha = alpha
        self.beta = beta
        self.random_state = check_random_state(random_state)
        self.n_init = n_init
        self.n_jobs = n_jobs
        self.init = init
        self.max_iter_outer_loop = max_iter_outer_loop
        self.verbose = verbose
        self.bias = bias

    def fit(self, counts):
        """

        """

        if not sparse.isspmatrix_coo(counts):
            counts = sparse.coo_matrix(counts)

        if self.init == "MDS2":
            X = mds.estimate_X(counts, alpha=self.alpha,
                               beta=self.beta,
                               ini="random",
                               verbose=self.verbose,
                               bias=self.bias,
                               random_state=self.random_state,
                               maxiter=self.max_iter)
        elif self.init == "random":
            X = self.init
        else:
            raise ValueError("Unknown initialization strategy")

        self.alpha_ = self.alpha
        self.beta_ = self.beta
        for it in range(self.max_iter_outer_loop):
            self.alpha_, self.beta_ = poisson_model.estimate_alpha_beta(
                counts,
                X, bias=self.bias, ini=[self.alpha_, self.beta_],
                verbose=self.verbose,
                random_state=self.random_state)
            print self.alpha_, self.beta_
            X_ = estimate_X(counts,
                            alpha=self.alpha_,
                            beta=self.beta_,
                            ini=X,
                            verbose=self.verbose,
                            bias=self.bias,
                            random_state=self.random_state,
                            maxiter=self.max_iter)
        return X_



import numpy as np
from . import solver
from . import utils


def estimate_X(counts, factr=0, init=None, alpha=-3, random_state=None, lengths=None,
                 verbose=None,
                 maxiter=100):
    mds = solver.MDS()
    if lengths is None:
        lengths = np.array([counts.shape[1]])
    wd = utils.compute_wish_distances(counts, alpha=alpha)
    mds.fit(wd, lengths, random_state=random_state, init=init)
    return mds.X_



import numpy as np
from scipy import sparse
import pandas as pd


def load_counts(filename, lengths=None):
    """
    Fast loading of a raw interaction counts file

    Parameters
    ----------
    filename : str
        path to the file to load. The file should be of the following format:
        i, j, counts

    Returns
    --------
    X : the interaction counts file
    """
    n = None
    # This is the interaction count files
    dataframe = pd.read_csv(filename, sep="\t", header=None)
    row, col, data = dataframe.as_matrix().T
    if lengths is not None:
        n = lengths.sum()
    else:
        n = max(row.max(), col.max()) + 1
    shape = (n, n)

    # XXX We need to deal with the fact that we should not duplicate entries
    # for the diagonal.
    # XXX what if n doesn't exist?
    if (col.min() >= 1 and row.min() >= 1) and \
       ((n is None) or (col.max() == n)):
        # This is a hack to deal with the fact that sometimes, the files are
        # indexed at 1 and not 0
        col -= 1
        row -= 1

    data = data.astype(float)
    counts = sparse.coo_matrix((data, (row, col)), shape=shape)
    return counts


def load_lengths(filename):
    """
    Fast loading of the bed files

    Parameters
    ----------
    filename : str,
        path to the file to load. The file should be a bed file

    Returns
    -------
    lengths : the lengths of each chromosomes
    """
    data = pd.read_csv(filename, sep="\t", comment="#", header=None)
    data = data.as_matrix()
    _, idx, lengths = np.unique(data[:, 0], return_counts=True,
                                return_index=True)
    return lengths[idx.argsort()]


def write_counts(filename, counts):
    """
    Write counts

    Parameters
    ----------

    filename : str

    counts: array-like
    """
    if not sparse.isspmatrix_coo(counts):
        if sparse.issparse(counts):
            counts = counts.tocoo()
        else:
            counts = sparse.coo_matrix(counts)
    # XXX this is slow and memory intensive
    data = np.concatenate([counts.row[:, np.newaxis],
                           counts.col[:, np.newaxis],
                           counts.data[:, np.newaxis]], axis=1)
    np.savetxt(filename, data, fmt="%d\t%d\t%f")


def write_lengths(filename, lengths, resolution=1):
    """
    Write lengths as bed file
    """
    chromosomes = ["Chr%02d" % (i + 1) for i in range(len(lengths))]
    j = 0
    with open(filename, "w") as bed_file:
        for chrid, l in enumerate(lengths):
            for i in range(l):
                bed_file.write(
                    "%s\t%d\t%d\t%d\n" % (chromosomes[chrid],
                                          i * resolution + 1,
                                          (i + 1) * resolution,
                                          j))
                j += 1
